import axios from 'axios';
const cheerio = require('cheerio')
import { downloadImage } from "../share/tools"
import { getCollection } from '../lib/mongodb';
import Turndown from 'turndown';
import { FetchType } from '..';
import { v4 as uuid } from 'uuid';
// 要抓取的网页 URL
function removeQueryParams(url: string): string {
  const urlObj = new URL(url);
  urlObj.search = ''; // 清空查询参数
  return urlObj.toString();
}

async function getPageData(url: string, name: string) {
  try {

    const { data } = await axios.get(url);
    const $ = cheerio.load(data);
    const element = $(`a[title="${name}"]`)
    const link = removeQueryParams(element.attr('href') as string)
    const panelBodyHtml = $('.panel-body').html();

    // 2. 使用Turndown将HTML转换为Markdown
    const turndown = new Turndown({
      codeBlockStyle: 'fenced', // 代码块用```包裹
      headingStyle: 'atx'       // 标题用#符号
    });

    // 添加自定义规则（可选）
    turndown.addRule('preCodeBlock', {
      filter: ['pre'],
      replacement: (content) => {
        // 保留pre标签内的原始格式（如代码块）
        return '\n```\n' + content + '\n```\n';
      }
    });
    // 执行转换
    const content = turndown.turndown(panelBodyHtml);
    const cover = await downloadImage($('.img-cover').attr('data-src'))
    const title = $('.site-name').text().trim()
    return {
      link,
      title,
      cover,
      content,
      _id: uuid()

    }
  } catch (error) {
    console.error('Error fetching data:', error);

  }
}
export async function queryListData(list: FetchType[]) {
  const col = await getCollection('link');
  await col.deleteMany({})
  for (const item of list) {
    await fetchData(item)
  }
}
export async function fetchData({ typeId, name: typeName, url, hasSubType = false }: FetchType) {
  try {
    // 请求目标页面
    const { data } = await axios.get(url);

    // 使用 cheerio 加载 HTML

    const $ = cheerio.load(data);

    // 提取工具卡片数据
    const toolsData = [] as any[];
    const articleDataList = [] as any[];
    const length = $('.url-card').length
    let i = 0
    for (const element of $('.url-card')) {
      const subTitle = $(element).parent().prev().find('h4').text().trim();
      const name = $(element).find('.url-info strong').text().trim();
      const tempLink = $(element).find('a').attr('href') as string;
      const description = $(element).find('.url-info p').text().trim();
      const _id = $(element).attr('data-id');
      const _originLink = $(element).find('img').attr('data-src');
      let link = removeQueryParams(tempLink)

      let articleData;
      console.log(subTitle);

      if (tempLink.startsWith('https://ai-bot')) {
        const pageData = await getPageData(tempLink, name)
        if (pageData) {
          link = pageData.link
          articleData = pageData
        }


      }
      // 假设工具的类别是固定的，比如 "AI写作工具"
      const type = typeId;
      const priority = 1;  // 根据索引来定义优先级
      const addTime = Math.floor(Date.now() / 1000);
      const logoLink = await downloadImage(_originLink)
      // console.log(logoLink);
      // const logoLink = ''
      // 将提取的数据转换为 Link 类型
      const toolData = {
        name,
        link,
        description,
        _id,
        type,
        priority,
        logoLink,
        addTime,
        articleId: articleData?._id ? articleData._id : undefined,
        subLinkType: hasSubType ? [subTitle] : undefined
      };
      if (articleData) {
        articleDataList.push({
          ...articleData,
          addTime: new Date().getTime() / 1000,
          priority: 0

        })
      }
      if (hasSubType) {
        if (toolsData.findIndex(val => val.name === name) !== -1) {
          console.log('发现相同的name:' + name);

          toolsData[toolsData.findIndex(val => val.name === name)].subLinkType.push(subTitle)
        } else {
          toolsData.push(toolData);

        }
      } else {
        toolsData.push(toolData);
      }

      i++
      // console.clear()
      console.log(`正在爬取${typeName + ',' + subTitle || ''}类别的数据，共${length}条数据`);

      console.log(`${typeName}：进度：${i}/${length}`);

    }
    console.log(toolsData);
    console.log(articleDataList);

    const col = await getCollection('link');
    await col.insertMany(toolsData);
    const articleCol = await getCollection('article');
    await articleCol.insertMany(articleDataList)
    console.log('数据插入成功');
  } catch (error) {
    console.error('Error fetching data:', error);
  }
}


